# Browser Copilot Test Suite

This directory contains the comprehensive test suite for Browser Copilot, set up using pytest.

## Overview

The test suite includes:
- **Unit Tests**: Testing individual modules in isolation
- **Integration Tests**: Testing component interactions
- **Test Coverage**: Using pytest-cov for coverage reporting

## Test Structure

```
tests/
├── conftest.py          # Pytest configuration and fixtures
├── test_storage_manager.py
├── test_config_manager.py
├── test_token_optimizer.py
├── test_io_handlers.py
├── test_verbose_logger.py
├── test_test_enhancer.py
├── test_reporter.py
└── test_integration.py
```

## Running Tests

### Run all tests
```bash
pytest tests/
```

### Run specific test file
```bash
pytest tests/test_storage_manager.py
```

### Run with verbose output
```bash
pytest tests/ -v
```

### Run with coverage
```bash
pytest tests/ --cov=browser_copilot --cov-report=term-missing
```

### Generate HTML coverage report
```bash
pytest tests/ --cov=browser_copilot --cov-report=html
```

### Run tests by marker
```bash
# Unit tests only
pytest tests/ -m unit

# Integration tests only  
pytest tests/ -m integration
```

## Test Fixtures

Common fixtures are defined in `conftest.py`:

- `temp_dir`: Provides a temporary directory for test files
- `mock_storage_dir`: Creates a mock storage directory structure
- `sample_test_suite`: Provides sample test content
- `sample_config`: Provides sample configuration
- `mock_llm`: Mock LLM for testing
- `mock_telemetry`: Mock telemetry callback
- `cleanup_env`: Cleans up environment variables after tests

## Current Test Status

As of the last run:
- **39 tests passing**
- **Coverage**: ~85-90% for tested modules (storage_manager, config_manager, token_optimizer)
- **Known Issues**: 
  - Some tests require mocking due to modelforge/jinja2 dependency conflicts
  - LangChain message optimization test needs fixing

## Writing New Tests

1. Create test file with `test_` prefix
2. Import necessary modules with the direct import pattern:
   ```python
   sys.path.insert(0, str(Path(__file__).parent.parent / 'browser_copilot'))
   from module_name import ClassName
   ```
3. Use appropriate markers (@pytest.mark.unit or @pytest.mark.integration)
4. Use fixtures from conftest.py where applicable
5. Follow existing test patterns for consistency

## Dependencies

Test dependencies are included in the main project requirements:
- pytest
- pytest-cov
- pytest-asyncio
- PyYAML (for YAML tests)

## Continuous Integration

The test suite is designed to be CI-friendly:
- All tests can run without external dependencies
- Tests use temporary directories to avoid conflicts
- Environment variables are cleaned up after each test
- Tests are properly marked for filtering